import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A graphical user interface for the calculator. No calculation is being
 * done here. This class is responsible just for putting up the display on 
 * screen. It then refers to the "CalcEngine" to do all the real work.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 31 July 2000
 *
 * Modified by Viera K. Proulx 
 * 21 July 2005
 */
public class UserInterface
{
    private CalcEngine calc;
    private boolean showingAuthor;

    private JFrame frame;
    private JTextField display;
    private JLabel status;

    /**
     * Create a user interface for a given calcEngine.
     */
    public UserInterface(CalcEngine engine)
    {
        calc = engine;
        showingAuthor = true;
        makeFrame();
        frame.setVisible(true);
    }

    /**
     * Make this interface visible again. (Has no effect if it is already
     * visible.)
     */
    public void setVisible(boolean visible)
    {
        frame.setVisible(visible);
    }

    /**
     * Make the frame for the user interface.
     */
    private void makeFrame()
    {
        frame = new JFrame(calc.getTitle());
		
	
        
        JPanel contentPane = (JPanel)frame.getContentPane();
        contentPane.setLayout(new BorderLayout(8, 8));
        contentPane.setBorder(new EmptyBorder( 10, 10, 10, 10));

        display = new JTextField();
        contentPane.add(display, BorderLayout.NORTH);

        JPanel buttonPanel = new JPanel(new GridLayout(4, 4));
            addNumberButton(buttonPanel, 7);
            addNumberButton(buttonPanel, 8);
            addNumberButton(buttonPanel, 9);
            addClearButton(buttonPanel);
            
            addNumberButton(buttonPanel, 4);
            addNumberButton(buttonPanel, 5);
            addNumberButton(buttonPanel, 6);
            addVersionButton(buttonPanel);
            
            addNumberButton(buttonPanel, 1);
            addNumberButton(buttonPanel, 2);
            addNumberButton(buttonPanel, 3);
            buttonPanel.add(new JLabel(" "));
            
            addNumberButton(buttonPanel, 0);

            addOperatorButton(buttonPanel, new Plus(), "+");
            addOperatorButton(buttonPanel, new Subtract(), "-");
            addEqualsButton(buttonPanel);
            
        contentPane.add(buttonPanel, BorderLayout.CENTER);

        status = new JLabel(calc.getAuthor());
        contentPane.add(status, BorderLayout.SOUTH);

        frame.pack();
    }

    /**
     * Add a button to the button panel.
     */
    private void addVersionButton(Container panel)
    {
        JButton button = new JButton("?");
        button.addActionListener(new ActionListener(){
		
			public void actionPerformed(ActionEvent event){
				showInfo();
				redisplay();
			}		
		});
        panel.add(button);
    }

    /**
     * Add a number button to the button panel.
     */
    private void addNumberButton(Container panel, int number)
    {
        JButton button = new JButton("" + number);
        button.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event){
				String command = event.getActionCommand();
				int number = Integer.parseInt(command);
				calc.numberPressed(number);
				redisplay();
			}
		});
        panel.add(button);
    }

    /**
     * Add an operator button to the button panel.
     */
    private void addOperatorButton(Container panel, Operator op, String label)
    {
        JButton button = new JButton(label);
        final Operator localOp = op;

        button.addActionListener(new ActionListener(){
		
			public void actionPerformed(ActionEvent event){
				calc.operation(localOp);
				redisplay();
			}		
		});
        panel.add(button);
    }
	
    /**
     * Add an equals button to the button panel.
     */
    private void addEqualsButton(Container panel)
    {
        JButton button = new JButton("=");
        button.addActionListener(new ActionListener(){
		
			public void actionPerformed(ActionEvent event){
				calc.equalsOp();
				redisplay();
			}		
		});
        panel.add(button);
    }

    /**
     * Add a clear button to the button panel.
     */
    private void addClearButton(Container panel)
    {
        JButton button = new JButton("C");
        button.addActionListener(new ActionListener(){
		
			public void actionPerformed(ActionEvent event){
				calc.clearOp();
				redisplay();
			}		
		});
        panel.add(button);
    }
	
    /**
     * Update the interface display to show the current value of the 
     * calculator.
     */
    private void redisplay()
    {
        display.setText("" + calc.getDisplayValue());
    }

    /**
     * Toggle the info display in the calculator's status area between the
     * author and version information.
     */
    private void showInfo()
    {
        if(showingAuthor)
            status.setText(calc.getVersion());
        else
            status.setText(calc.getAuthor());

        showingAuthor = !showingAuthor;
    }
}
